package com.ruoyi.zdsz.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.zdsz.domain.ZEngineeringIndustry;
import com.ruoyi.zdsz.domain.bo.ZEngineeringIndustryBo;
import com.ruoyi.zdsz.domain.bo.ZEngineeringNodeBo;
import com.ruoyi.zdsz.domain.bo.ZEngineeringReviewBo;
import com.ruoyi.zdsz.domain.vo.ZEngineeringIndustryVo;
import com.ruoyi.zdsz.domain.vo.ZEngineeringReviewVo;
import com.ruoyi.zdsz.mapper.ZEngineeringIndustryMapper;
import com.ruoyi.zdsz.service.IZEngineeringIndustryService;
import com.ruoyi.zdsz.service.IZEngineeringNodeService;
import com.ruoyi.zdsz.service.IZEngineeringReviewService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 工业工程|市政工程Service业务层处理
 *
 * @author ruoyi
 * @date 2024-01-02
 */
@RequiredArgsConstructor
@Service
public class ZEngineeringIndustryServiceImpl implements IZEngineeringIndustryService {

    @Resource
    private final ZEngineeringIndustryMapper baseMapper;

    @Resource
    private IZEngineeringNodeService izEngineeringNodeService;

    @Autowired
    private IZEngineeringNodeService nodeService;

    @Resource
    private IZEngineeringReviewService izEngineeringReviewService;
    /**
     * 查询工业工程|市政工程
     */
    @Override
    public ZEngineeringIndustryVo queryById(String id) {
        ZEngineeringIndustryVo zEngineeringIndustryVo = baseMapper.selectVoById(id);
        ZEngineeringNodeBo zEngineeringNodeBo = new ZEngineeringNodeBo();
        zEngineeringNodeBo.setCivliId(id);
        zEngineeringNodeBo.setCreateTime(zEngineeringIndustryVo.getCreateTime());
        List<ZEngineeringNodeBo> zEngineeringNodeBos = izEngineeringNodeService.queryListDetails(zEngineeringNodeBo);
        zEngineeringIndustryVo.setZEngineeringNodeBo(zEngineeringNodeBos);
        return zEngineeringIndustryVo;

    }

    /**
     * 查询工业工程|市政工程列表
     */
    @Override
    public TableDataInfo<ZEngineeringIndustryVo> queryPageList(ZEngineeringIndustryBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<ZEngineeringIndustry> lqw = buildQueryWrapper(bo);
        Page<ZEngineeringIndustryVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询工业工程|市政工程列表
     */
    @Override
    public List<ZEngineeringIndustryVo> queryList(ZEngineeringIndustryBo bo) {
        LambdaQueryWrapper<ZEngineeringIndustry> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<ZEngineeringIndustry> buildQueryWrapper(ZEngineeringIndustryBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<ZEngineeringIndustry> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getEnginType()), ZEngineeringIndustry::getEnginType, bo.getEnginType());
        lqw.like(StringUtils.isNotBlank(bo.getConstructUnit()), ZEngineeringIndustry::getConstructUnit, bo.getConstructUnit());
        lqw.eq(StringUtils.isNotBlank(bo.getProjectHead()), ZEngineeringIndustry::getProjectHead, bo.getProjectHead());
        lqw.eq(StringUtils.isNotBlank(bo.getSceneHead()), ZEngineeringIndustry::getSceneHead, bo.getSceneHead());
        lqw.eq(StringUtils.isNotBlank(bo.getDesignUnit()), ZEngineeringIndustry::getDesignUnit, bo.getDesignUnit());
        lqw.eq(StringUtils.isNotBlank(bo.getDesignHead()), ZEngineeringIndustry::getDesignHead, bo.getDesignHead());
        lqw.eq(StringUtils.isNotBlank(bo.getDesignPhone()), ZEngineeringIndustry::getDesignPhone, bo.getDesignPhone());
        lqw.eq(StringUtils.isNotBlank(bo.getSupervisionUnit()), ZEngineeringIndustry::getSupervisionUnit, bo.getSupervisionUnit());
        lqw.eq(StringUtils.isNotBlank(bo.getSupervisionHead()), ZEngineeringIndustry::getSupervisionHead, bo.getSupervisionHead());
        lqw.eq(StringUtils.isNotBlank(bo.getSupervisionPhone()), ZEngineeringIndustry::getSupervisionPhone, bo.getSupervisionPhone());
        lqw.eq(StringUtils.isNotBlank(bo.getType()), ZEngineeringIndustry::getType, bo.getType());
        return lqw;
    }

    /**
     * 新增工业工程|市政工程
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean insertByBo(ZEngineeringIndustryBo bo) {
        ZEngineeringIndustry add = BeanUtil.toBean(bo, ZEngineeringIndustry.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        bo.getzEngineeringNodeBo().setCivliId(add.getId());
        bo.getzEngineeringNodeBo().setCreateTime(new Date());
        izEngineeringNodeService.insert(bo.getzEngineeringNodeBo());
        return flag;
    }

    /**
     * 修改工业工程|市政工程
     */
    @Override
    public Boolean updateByBo(ZEngineeringIndustryBo bo) {
        ZEngineeringIndustry update = BeanUtil.toBean(bo, ZEngineeringIndustry.class);
        validEntityBeforeSave(update);
        boolean flag = baseMapper.updateById(update) > 0;
        bo.getzEngineeringNodeBo().setCivliId(bo.getId());
        bo.getzEngineeringNodeBo().setType(bo.getType());
        izEngineeringNodeService.update(bo.getzEngineeringNodeBo());
        return flag;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(ZEngineeringIndustry entity) {
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除工业工程|市政工程
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<String> ids, Boolean isValid) {
        if (isValid) {
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        ids.forEach(item -> {
            ZEngineeringIndustryVo vo = baseMapper.selectVoById(item);
            ZEngineeringNodeBo zEngineeringNodeBo = new ZEngineeringNodeBo();
            zEngineeringNodeBo.setCreateTime(vo.getCreateTime());
            zEngineeringNodeBo.setCivliId(vo.getId());
            List<ZEngineeringNodeBo> zEngineeringNodeBos = nodeService.queryListDetails(zEngineeringNodeBo);
            zEngineeringNodeBos.stream().forEach(ite ->
                nodeService.delete(ite)
            );
        });
        return baseMapper.deleteBatchIds(ids) > 0;
    }

    //审核工业工程|市政工程
    @Override
    public Boolean auditById(String id) {
        Boolean flag = false;
        ZEngineeringNodeBo bo = new ZEngineeringNodeBo();
        bo.setCivliId(id);
        List<ZEngineeringNodeBo> zEngineeringNodeBos = izEngineeringNodeService.queryListDetails(bo);
        //根据节点的id查询审核状态
        zEngineeringNodeBos.stream().forEach(item -> {
            ZEngineeringReviewBo zEngineeringReviewBo = new ZEngineeringReviewBo();
            zEngineeringReviewBo.setEngInfoId(item.getId());
            List<ZEngineeringReviewVo> vos = izEngineeringReviewService.queryList(zEngineeringReviewBo);
            //判断审核状态
            vos.stream().forEach(item1 -> {
                if (item1.getReviewStatus().equals("0")) {
                }
            });
        });
        return null;
    }



}
